#include "../../stdafx.h"

#include "Console.h"
#include <time.h>

using namespace con;


CConsole* CConsole::mSingleInstance = 0;


//---------------------------------------------------------------------------
// CConsole
//---------------------------------------------------------------------------

CConsole::CConsole()
:	mDefaultRenderingCtx( NULL ),
	mDefaultFilter( NULL )
{
}

CConsole::CConsole( IConsoleRenderingContext& ctx )
:	mDefaultRenderingCtx( &ctx ),
	mDefaultFilter( NULL )
{
}

CConsole::CConsole( IConsoleRenderingContext& ctx, IFilter& filter )
:	mDefaultRenderingCtx( &ctx ),
	mDefaultFilter( &filter )
{
}

CConsole::~CConsole()
{
	for( TChannelList::iterator i = mChannels.begin(); i != mChannels.end(); ++i )
		delete *i;
}

CChannel& CConsole::openChannel( const std::string& name )
{
	CChannel* channel = NULL;

	for( TChannelList::iterator i = mChannels.begin(); i != mChannels.end(); ++i ) {
		if( (*i)->getName() == name )
			channel = *i;
	}

	if( !channel ) {
		channel = new CChannel( *this, name, mDefaultRenderingCtx, mDefaultFilter );
		mChannels.push_back( channel );
	}

	return *channel;
}

void CConsole::closeChannel( CChannel& channel )
{
	for( TChannelList::iterator i = mChannels.begin(); i != mChannels.end(); ++i ) {
		if( *i == &channel ) {
			delete *i;
			i = mChannels.erase( i );
		}
	}
}

void CConsole::setDefaultRenderingContext( IConsoleRenderingContext& ctx )
{ 
	mDefaultRenderingCtx = &ctx;

	for( TChannelList::iterator i = mChannels.begin(); i != mChannels.end(); ++i ) {
		if( !(*i)->getRenderingContext() )
			(*i)->setRenderingContext( ctx );
	}
}


//---------------------------------------------------------------------------
// CChannel
//---------------------------------------------------------------------------

CChannel::CChannel( CConsole& console, const std::string& name, IConsoleRenderingContext* renderingCtx, IFilter* filter )
:	mParentConsole( console ),
	mName( name ),
	mRenderingCtx( renderingCtx ),
	mFilter( filter ),
	mWritingChannel( true ),
	mLoggingTime( false )
{
}

CChannel::~CChannel()
{
}

void CChannel::write( const std::string& methodName, const std::string& message )
{
	write( message );
}

void CChannel::write( const std::string& message )
{
	if( mFilter && !mFilter->apply( *this, message ) )
		return;

	if( mLoggingTime ) {
		time_t clockTime;
		time( &clockTime );
		const tm* t = localtime( &clockTime );
		char buf[100];
		sprintf( buf, "%i:%i:%i ", t->tm_hour, t->tm_min, t->tm_sec );
		mRenderingCtx->write( buf );
	}
	if( mWritingChannel ) {
		mRenderingCtx->write( mName + "> " );
	}
	mRenderingCtx->write( message );
	mRenderingCtx->write( "\n" );
}
