#include "../../stdafx.h"

#include "Wheel.h"
#include "../MyDemo.h"
#include "../../engine/camera/Camera.h"
//#include "../../utils/Random.h"


// ------------------------------------------------------------------
//  basic stuff


CWheelEntity::CWheelEntity( const CCamera& camera )
:	mLineRenderer( NULL ),
	mLinePool( MAX_LINES )
{
	mLineRenderer = new CLineRenderer( camera, CResourceId("AlphaParticle.sha"), CResourceId("lineHard.png"), mLinePool );
	assert( mLineRenderer );
	CMyDemo::getInstance().getObjects().addObject( *mLineRenderer );
	mLineRenderer->setVisible( false );
}

CWheelEntity::~CWheelEntity()
{
	CMyDemo::getInstance().getObjects().removeObject( *mLineRenderer );
	delete mLineRenderer;
}


bool CWheelEntity::isActive( CScenario::eScene scene ) const
{
	return scene >= CScenario::LINES_CIRCLE_BLOB && scene <= CScenario::FADEWHEEL_QUAD_BLOB_LINES;
}

void CWheelEntity::onSceneStart( CScenario::eScene scene )
{
	if( scene == CScenario::LINES_CIRCLE_BLOB ) {
		mLineRenderer->setVisible(true);
		mLinePool.clear();

		mRadius = 30.0f;
		mWidth = 2.0f;
		mHoleAmount = 0.0f;
		mRot = 0.0f;

		// lines
		int i;
		for( i = 0; i < MAX_LINES; ++i ) {
			SLine& l = mLinePool.add();
			l.info.color = (DWORD)0xFFFFFFFF;
		}
		setupWheel();
		for( i = 0; i < MAX_LINES; ++i ) {
			SLine& l = mLinePool.get(i);
			l.beforeUpdate();
		}
	}
}

void CWheelEntity::onSceneEnd( CScenario::eScene scene )
{
	if( scene == CScenario::FADEWHEEL_QUAD_BLOB_LINES ) {
		mLineRenderer->setVisible(false);
	}
}

void CWheelEntity::setupWheel()
{
	const int HOLES = 4;

	int n = mLinePool.size();
	int spann = n/HOLES;

	float twopi = 2.0f * 3.1415927f;
	float spanf = twopi / HOLES;

	mRot = fmodf(mRot,twopi);

	int i = 0;
	for( int q = 0; q < HOLES; ++q ) {
		float f = spanf * q;
		float df = (spanf-mHoleAmount) / spann;
		float df1 = df*1.2f;
		for( int j = 0; j < spann; ++j, f += df, ++i ) {
			SLine& l = mLinePool.get(i);
			l.beforeUpdate();

			l.info.endA.x = cosf(f+mRot)*mRadius;
			l.info.endA.y = sinf(f+mRot)*mRadius;
			l.info.endA.z = 100.0f;
			l.info.endB.x = cosf(f+mRot+df1)*mRadius;
			l.info.endB.y = sinf(f+mRot+df1)*mRadius;
			l.info.endB.z = 100.0f;
			l.info.halfWidth = mWidth;
		}
	}
}

void CWheelEntity::update()
{
	switch( CScenario::getInstance().getCurrScene() ) {
	case CScenario::LINES_CIRCLE_BLOB:
	case CScenario::CIRCLE_BLOB:
		updateSceneCircle(); break;
	case CScenario::WHEEL_BLOB:
	case CScenario::WHEEL_QUAD_BLOB:
	case CScenario::WHEEL_QUAD_BLOB_LINES:
		updateSceneWheel(); break;
	case CScenario::FADEWHEEL_QUAD_BLOB_LINES:
		updateSceneFade(); break;
	}
}

void CWheelEntity::updateSceneCircle()
{
	const CScenario& scenario = CScenario::getInstance();
	float timeStart = scenario.getSceneStart( CScenario::LINES_CIRCLE_BLOB );
	float timeEnd = scenario.getSceneStart( CScenario::WHEEL_BLOB );
	float alpha = (scenario.getCurrentTime()-timeStart) / (timeEnd-timeStart);

	if( scenario.getCurrScene() != CScenario::LINES_CIRCLE_BLOB ) {
		mRadius = 30.0f + scenario.getCurrSceneAlpha() * 50.0f;
		mWidth = 2.0f + scenario.getCurrSceneAlpha() * 4.0f;
	}
	setupWheel();
}

void CWheelEntity::updateSceneWheel()
{
	const CScenario& scenario = CScenario::getInstance();

	CScenario::eScene scene = scenario.getCurrScene();
	if( scene == CScenario::WHEEL_BLOB ) {
		mHoleAmount = scenario.getCurrSceneAlpha() * 0.2f;
	} else {
		float timeStart = scenario.getSceneStart( CScenario::WHEEL_QUAD_BLOB );
		float timeEnd = scenario.getSceneStart( CScenario::FADEWHEEL_QUAD_BLOB_LINES );
		float alpha = (scenario.getCurrentTime()-timeStart) / (timeEnd-timeStart);
		mRadius = 80.0f - alpha*50.0f;
		mRot = alpha * 10.0f + CMyDemo::getInstance().getPerlin().get1D( alpha ) * 1.0f;
		mWidth = 6.0f - alpha*5.0f;
	}
	setupWheel();
}

void CWheelEntity::updateSceneFade()
{
	const CScenario& scenario = CScenario::getInstance();
	float alpha = scenario.getCurrSceneAlpha();
	mRadius += 0.1f;
	mRot -= 0.05f;
	mWidth += 0.3f;
	setupWheel();

	// fade
	float transp = 1.0f - alpha;
	int n = mLinePool.size();
	for( int i = 0; i < n; ++i ) {
		SLine& l = mLinePool.get(i);
		l.info.color.a = transp;
	}
}
